<?php

namespace Drupal\dismissible_message_bar\Service;

use DateTimeZone;
use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Render\Renderer;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\TypedData\Exception\MissingDataException;
use Drupal\Core\Url;
use Exception;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class DmbNotificationService.
 *
 * @package Drupal\dismissible_message_bar\Service
 */
class DmbNotificationService {

  use StringTranslationTrait;

  /**
   * Current path stack.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPathStack;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Renderer service.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * Language Manager interface.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The path matcher.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * An alias manager to find the alias for the current system path.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * Constructs a new NotificationsService object.
   *
   * @param \Drupal\Core\Path\CurrentPathStack $current_path_stack
   *   Current path stack.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   Request Stack.
   * @param \Drupal\Core\Render\Renderer $renderer
   *   Renderer service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language manager service.
   * @param \Drupal\Core\Path\PathMatcherInterface $path_matcher
   *   Path matcher service.
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   Alias manager service.
   */
  public function __construct(
    CurrentPathStack $current_path_stack,
    EntityTypeManagerInterface $entity_type_manager,
    RequestStack $request_stack,
    Renderer $renderer,
    LanguageManagerInterface $language_manager,
    PathMatcherInterface $path_matcher,
    AliasManagerInterface $alias_manager) {

    $this->currentPathStack = $current_path_stack;
    $this->entityTypeManager = $entity_type_manager;
    $this->requestStack = $request_stack;
    $this->renderer = $renderer;
    $this->languageManager = $language_manager;
    $this->pathMatcher = $path_matcher;
    $this->aliasManager = $alias_manager;
  }

  /**
   * Returns array of all notifications.
   *
   * @return array
   *   Notification values to insert into drupal settings.
   */
  public function returnAllNotifications() {
    $result = [];

    try {
      $storage = $this->entityTypeManager->getStorage('dmb_notifications_entity');
      $query = $storage->getQuery()
        ->sort('created', 'DESC');

      $notifications = $query->execute();
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
      // Unable to load the notifications, quietly die.
      $notifications = NULL;
    }


    if (!empty($notifications) && isset($storage)) {
      $notifications = $storage->loadMultiple($notifications);

      foreach ($notifications as $notification) {
        // We're letting JS decide the date/time of the notification.
        // By default we display all notifications, and only remove
        // notifications from more than 24 hours ago.
        // OR notifications that do not start for more than 24 hours.
        $display = TRUE;

        // If the notification does not have a date range, set the start to be
        // 1 day ago and the end 1 day from now.
        $date_range = $notification->get('field_notification_date_range')->first();

        $gmtTimezone = new DateTimeZone('GMT');

        $now_time = new DrupalDateTime('now', $gmtTimezone);
        $now = $now_time->getTimestamp();

        if (!empty($date_range->value)) {
          $start_date = new DrupalDateTime($date_range->value, $gmtTimezone);
          $date_range_begin = $start_date->getTimestamp();

          // Filter notifications that start more than a day from now.
          if (($date_range_begin - $now) > 86400) {
            $display = FALSE;
          }
        }

        if (!empty($date_range->end_value)) {
          $end_date = new DrupalDateTime($date_range->end_value, $gmtTimezone);
          $date_range_end = $end_date->getTimestamp();

          // Filter notifications that end more than a day ago.
          // This range should prevent most timezone-related mess.
          if (($now - $date_range_end) > 86400) {
            $display = FALSE;
          }
        }


        if ($display) {
          // Build the view of referenced content.
          if (!isset($notification->field_p_content)) {
            // This notification does not have content, ignore it.
            continue;
          }

          $content = $notification->field_p_content;
          $viewBuilder = $this->entityTypeManager->getViewBuilder('paragraph');
          $content = $viewBuilder->viewField($content, ['label' => 'hidden']);

          $notification_id = $notification->id();

          // Automatically dismiss bar after set time.
          if ($notification->hasField('field_auto_dismiss') && !$notification->get('field_auto_dismiss')->isEmpty()) {
            $auto_dismiss = $notification->get('field_auto_dismiss')->value;
            if ($notification->hasField('field_dismiss_time') && !$notification->get('field_dismiss_time')->isEmpty()) {
              $dismiss_time = $notification->get('field_dismiss_time')->value;
            }
          }

          // Get the cookie expiration information.
          if ($notification->hasField('field_cookie_expiration') && !$notification->get('field_cookie_expiration')->isEmpty()) {
            $cookie_expiration = $notification->get('field_cookie_expiration')->value;
          }

          // Set the notification's classes.
          $classes = [
            'dmb-notification',
          ];
          // If notification has cookie turned off, set class.
          if ($notification->hasField('field_cookie_off') && !$notification->get('field_cookie_off')->isEmpty()) {
            $classes[] = 'cookie-off';
          }
          // Get the notification type (from term reference), set class.
          if ($notification->hasField('field_notification_type') && !$notification->get('field_notification_type')->isEmpty()) {
            $classes[] = strtolower(str_replace(' ', '-', $notification->get('field_notification_type')->entity->name->value));
          }

          $rendered_content = [
            '#type' => 'container',
            '#attributes' => [
              'class' => $classes,
              'dmb-notification-id' => $notification_id,
              'dmb-cookie-expiration' => $cookie_expiration ?? 365,
              'dmb-auto-dismiss' => $auto_dismiss ?? FALSE,
              'dmb-dismiss-time' => $dismiss_time ?? 0,
            ],
            'notification' => [
              '#type' => 'container',
              '#attributes' => [
                'class' => ["dmb-notification-content"],
              ],
              'content' => $content,
            ],
            'close_notification' => [
              '#title' => $this->t('Close notification'),
              '#type' => 'link',
              '#url' => Url::fromUserInput('#'),
              '#attributes' => [
                'class' => ["close-dmb-notification"],
              ],
            ],
          ];
          $rendered_content = $this->renderer->renderPlain($rendered_content);

          // Get allowed content types.
          $allowed_content_types = [];
          if ($notification->hasField('field_content_types') && !$notification->get('field_content_types')->isEmpty()) {
            foreach ($notification->get('field_content_types') as $content_type_ref) {
              $allowed_content_types[] = $content_type_ref->target_id;
            }
          }

          // Get sitewide notifications.
          $sitewide = '';
          if ($notification->hasField('field_sitewide') && !$notification->get('field_sitewide')->isEmpty()) {
            $sitewide = $notification->get('field_sitewide')->value;
          }

          // Get allowed pages.
          $allowed_pages = [];
          if ($notification->hasField('field_notification_pages') && !$notification->get('field_notification_pages')->isEmpty()) {
            $allowed_pages_raw = $notification->get('field_notification_pages')->value;

            $allowed_pages_raw = explode(PHP_EOL, $allowed_pages_raw);
            foreach ($allowed_pages_raw as $allowed_page) {
              $allowed_page = trim($allowed_page);
              // Trim off trailing slashes unless this is the front page.
              $allowed_page = $allowed_page === '/' ? $allowed_page : rtrim($allowed_page, '/');
              // Make sure the page has a leading slash.
              // JS window.location.pathname always has a leading slash.
              if (substr($allowed_page, 0, 1) === '!' && substr($allowed_page, 1, 1) !== '/') {
                $allowed_page = '!/' . substr($allowed_page, 1);
              }
              elseif (substr($allowed_page, 0, 1) !== '/' && substr($allowed_page, 0, 1) !== '!') {
                $allowed_page = '/' . $allowed_page;
              }
              $allowed_pages[] = $allowed_page;
            }

          }

          $start_time = isset($date_range->value) ? $date_range->value . '+00:00' : '';
          $end_time = isset($date_range->end_value) ? $date_range->end_value . '+00:00' : '';
          $result[$notification_id] = [
            'content' => $rendered_content,
            'startTime' => $start_time,
            'endTime' => $end_time,
            'contentTypes' => $allowed_content_types,
            'sitewide' => $sitewide,
            'pathLimit' => $allowed_pages,
          ];

        }
      }

    }

    return $result;
  }

}
